local playerClass = select(2,UnitClass("player"))
if playerClass ~= "ROGUE" then return end

if not DrDamage then DrDamage = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "AceHook-2.1", "FuBarPlugin-2.0") end
local L = AceLibrary("AceLocale-2.2"):new("DrDamage")
local BS = AceLibrary("Babble-Spell-2.2")
local SEA = AceLibrary("SpecialEvents-Aura-2.0")
local GT = AceLibrary("Gratuity-2.0")
local Deformat = AceLibrary("Deformat-2.0")

--General
local type = type
local pairs = pairs
local ipairs = ipairs
local tonumber = tonumber
local math_abs = math.abs
local math_floor = math.floor
local math_ceil = math.ceil
local string_match = string.match
local select = select
local UnitLevel = UnitLevel

--Melee
local GetCritChance = GetCritChance
local GetCombatRatingBonus = GetCombatRatingBonus
local GetItemInfo = GetItemInfo
local GetInventoryItemLink = GetInventoryItemLink
local GetInventorySlotInfo = GetInventorySlotInfo

local UnitRangedDamage = UnitRangedDamage
local UnitDamage = UnitDamage
local UnitAttackSpeed = UnitAttackSpeed
local UnitAttackPower = UnitAttackPower
local UnitRangedAttackPower = UnitRangedAttackPower

local function DrD_ClearTable( table )
	for k in pairs( table ) do
		table[k] = nil
	end
end

local function DrD_Round(x, y)
	return math_floor( x * 10 ^ y + 0.5 ) / 10 ^ y
end

local function DrD_MatchData( data, ... )
	if not data or not ... then
		return false
	end

	if type( data ) == "table" then
		for _, dataName in ipairs( data ) do
			for i = 1, select('#', ...) do
				if dataName == select(i, ...) then
					return true
				end
			end
		end
	else	
		for i = 1, select('#', ...) do
			if data == select(i, ...) then
				return true
			end
		end
	end
	
	return false
end

DrDamage.defaults = {
	FontEffect = "OUTLINE",
	FontSize = 10,
	FontXPosition = 0,
	FontYPosition = 0,
	FontColorDmg = { r = 1, g = 1, b = 0.2 },
}

local settings
function DrDamage:Melee_OnEnable()
	self:Melee_CheckBaseStats()
	self:Melee_OnProfileEnable()	
	
	DrDamage.options = {
		type='group',
		args = {
			ActionBar = {
				type = "group", desc = L["Options for the actionbar"], name = L["ActionBar"],
				args = {				
					FontSize = {
						type = 'range',
						name = L["Actionbar font size"],
						desc = L["Set the font size"],
						min = 6,
						max = 20,
						step = 1,
						order = 75,
						get =  function() return settings["FontSize"] end,
						set = 	function(v) 
								settings.FontSize = v
								self.visualChange = true
								self:UpdateAB()
							end,
					},
					FontEffect = {
						type = "text",
						name = L["Actionbar font effect"],
						desc = L["Set the font effect"],
						validate = { "Outline", "ThickOutline", "None" },
						order = 77,
						get =	function()
								if settings.FontEffect == "" then
									return "None"
								elseif settings.FontEffect == "THICKOUTLINE" then
									return "ThickOutline"
								else
									return "Outline"
								end
							end,
						set = 	function(v)
								if v == "None" then
									settings.FontEffect = ""
								elseif v == "ThickOutline" then
									settings.FontEffect = "THICKOUTLINE"
								else
									settings.FontEffect = "OUTLINE"
								end

								DrDamage.visualChange = true
								self:UpdateAB()
							end,
					},				
					FontXPosition = {
						type = 'range',
						name = L["Actionbar font horizontal position"],
						desc = L["Set the font horizontal position"],
						min = -5,
						max = 5,
						step = 1,
						order = 80,
						get =  function() return settings["FontXPosition"] end,
						set = 	function(v) 
								settings.FontXPosition = v
								self.visualChange = true
								self:UpdateAB()
							end,
					},
					FontYPosition = {
						type = 'range',
						name = L["Actionbar font vertical position"],
						desc = L["Set the font vertical position"],
						min = -15,
						max = 30,
						step = 1,
						order = 85,
						get =  function() return settings["FontYPosition"] end,
						set = 	function(v) 
								settings.FontYPosition = v
								self.visualChange = true
								self:UpdateAB()
							end,
					},
					FontColorDmg = {
						type = "text",
						name = L["Damage text color"],
						desc = L["Set the actionbar damage text color"],
						usage = L["<r=n, g=n, b=n> where n is 0-1"],
						order = 90,
						get = function()
							return "r="..settings.FontColorDmg.r..", g="..settings.FontColorDmg.g..", b="..settings.FontColorDmg.b
						end,
						set = function(colors)
							for k, v in string.gmatch(string.lower(colors), "([r|g|b])=([%d\.]+)") do
								if tonumber(v) >= 0 and tonumber(v) <= 1 then
									settings.FontColorDmg[k] = v
								end
							end
							self.visualChange = true
							self:UpdateAB()						
						end
					},
				},
			},
		},
	}
end

function DrDamage:Melee_OnProfileEnable()
	settings = self.db.profile
end

local normalizationTable = {
	["Daggers"] = 1.7,
	["One-Handed Axes"] = 2.4,
	["One-Handed Maces"] = 2.4,
	["One-Handed Swords"] = 2.4,
	["Fist Weapons"] = 2.4,	
	["Bows"] = 2.8,
	["Crossbows"] = 2.8,
	["Guns"] = 2.8,
	["Two-Handed Axes"] = 3.3,
	["Two-Handed Maces"] = 3.3,
	["Two-Handed Swords"] = 3.3,	
	["Polearms"] = 3.3,
	["Staves"] = 3.3,
	["Fishing Pole"] = 3.3
}

local oldValues = 0
function DrDamage:Melee_CheckBaseStats()
	local newValues = 
	GetCombatRatingBonus(6)
	+ GetCombatRatingBonus(7)
	+ GetCombatRatingBonus(18)
	+ GetCombatRatingBonus(19)	
	+ self:GetAP() 
	+ self:GetRAP() 
	+ GetCritChance()
	+ GetRangedCritChance()

	if newValues ~= oldValues then
		oldValues = newValues
		return true
	end

	return false	
end

function DrDamage:Melee_InventoryChanged( unit )
	if self:Melee_CheckBaseStats() then
		return true	
	end
end

function DrDamage:GetAP()
	local baseAP, posBuff, negBuff = UnitAttackPower("player")
	return baseAP + posBuff + negBuff
end

function DrDamage:GetRAP()
	local baseAP, posBuff, negBuff = UnitRangedAttackPower("player")
	return baseAP + posBuff + negBuff	
end

function DrDamage:GetWeaponType()
	local mainLink = GetInventoryItemLink("player",GetInventorySlotInfo("MainHandSlot"))
	local offLink = GetInventoryItemLink("player",GetInventorySlotInfo("SecondaryHandSlot"))
	local mainH, offH
	if mainLink then
		mainH = select(7,GetItemInfo(mainLink))
	else
		mainH = "None"
	end
	if offLink then
		offH = select(7,GetItemInfo(offLink))
	else
		offH = "None"
	end
	return mainH, offH
end

function DrDamage:GetNormM()
	local mainH, offH = self:GetWeaponType()
	return normalizationTable[mainH], normalizationTable[offH]
end

function DrDamage:BaseDamage(AP,ranged)
	local min, max, omin, omax, mod, spd, ospd
	if ranged then
		spd, min, max, _, _, mod = UnitRangedDamage("player")
	else
		min, max, omin, omax, _, _, mod = UnitDamage("player")
		spd, ospd = UnitAttackSpeed("player")	
	end
	
	min = min / mod - (AP / 14) * spd
	max = max / mod - (AP / 14) * spd
		
	if not ranged and OffhandHasWeapon() then
		omin = math.floor((omin/0.5)/mod - (AP / 14) * ospd + 0.5)
		omax = math.floor((omax/0.5)/mod - (AP / 14) * ospd + 0.5)
	end
	
	return math.floor(min+0.5), math.floor(max+0.5), omin, omax, mod
end

function DrDamage:WeaponDamage(AP, ws, ranged)
	local min, max, omin, omax, mod = DrDamage:BaseDamage(AP, ranged)
	local normM
	
	if ranged then
		normM = UnitRangedDamage("player")
	else
		normM, onormM = UnitAttackSpeed("player")
	end
	
	if not ws then
		mh, oh = DrDamage:GetNormM()
		normM = mh or normM
		onormM = oh or onormM
	end
	
	min = (min + (AP / 14) * normM) * mod
	max = (max + (AP / 14) * normM) * mod
	
	if onormM and omin and omax then
		omin = (omin + (AP / 14) * onormM) * 0.5 * mod
		omax = (omax + (AP / 14) * onormM) * 0.5 * mod
	end
	
	return min, max, omin, omax
end

local ActiveAuras = {}
local BuffTalentRanks = {}
local CalculationResults = {}

local calculation = {}

local DrD_TalentCalc

function DrDamage:MeleeCalc( action, lActionName, tooltip )
	if not action or not lActionName then
		do return end
	end
	
	DrD_ClearTable( calculation )
	
	local actionName = lActionName
	
	if BS and BS:HasReverseTranslation(actionName) then
		actionName = BS:GetReverseTranslation(actionName)
	end
	
	local baseAction = DrDamage.spellInfo[lActionName][0]
	local ranged, AP
	calculation.dmgM = 1
	calculation.eDuration = action.eDuration or baseAction.eDuration or 0
	calculation.WeaponDamage = action.WeaponDamage or baseAction.weaponDamage
	
	if type( baseAction.School ) == "table" then
		calculation.actionSchool = baseAction.School[1]
		calculation.actionType = baseAction.School[2]
	else
		calculation.actionSchool = baseAction.School or "Physical"	
	end
	 
	if calculation.actionSchool == "Ranged" then
		ranged = true
		AP = DrDamage:GetRAP()
		calculation.critPerc = GetRangedCritChance()
		calculation.hitPerc = GetCombatRatingBonus(7)
		calculation.haste = GetCombatRatingBonus(18)
	else
		AP = DrDamage:GetAP()
		calculation.critPerc = GetCritChance()
		calculation.hitPerc = GetCombatRatingBonus(6)
		calculation.haste = GetCombatRatingBonus(19)
	end
	
	--Apply talents		
	for talentName, talentRank in pairs(self.talents) do						
		local talentTable = DrDamage.talentInfo[talentName]
		
		for i = 1, #talentTable do
			local talent = talentTable[i]	

			if DrD_MatchData(talent.Spells, actionName, calculation.actionTypeType) or DrD_MatchData(talent.Spells, "All", calculation.actionSchool) then		
				DrD_TalentCalc(baseAction, talentName, talentRank, talent)
			end
		end
	end	
	
	if calculation.WeaponDamage then
		local min, max, omin, omax = DrDamage:WeaponDamage( AP, baseAction.NoNormalization, ranged )
		min = min * calculation.WeaponDamage
		max = max * calculation.WeaponDamage
		if baseAction.DualAttack and OffhandHasWeapon() then
			min = min + omin * calculation.WeaponDamage + action[1]
			max = max + omax * calculation.WeaponDamage + (action[2] or action[1])
		end
		calculation.minDam, calculation.maxDam = min, max
	else
		calculation.minDam = 0
		calculation.maxDam = 0
	end	
	
	calculation.minDam = calculation.minDam + action[1]
	calculation.maxDam = calculation.maxDam + (action[2] or action[1])
	
	if baseAction.ComboPoints then
		local cp = GetComboPoints()
		
		if cp > 0 then
			if action.PerCombo then
				calculation.minDam = calculation.minDam + action.PerCombo * ( cp - 1 )
				calculation.maxDam = calculation.maxDam + action.PerCombo * ( cp - 1 )
			end
			if baseAction.ExtraPerCombo then
				calculation.minDam = calculation.minDam + action.Extra * baseAction.ExtraPerCombo[cp]
				calculation.maxDam = calculation.maxDam + action.Extra * baseAction.ExtraPerCombo[cp]
			end
			if baseAction.APBonus then
				if type( baseAction.APBonus ) == "table" then
					calculation.minDam = calculation.minDam + baseAction.APBonus[cp] * AP
					calculation.maxDam = calculation.minDam + baseAction.APBonus[cp] * AP
				else
					calculation.minDam = calculation.minDam + baseAction.APBonus * cp * AP
					calculation.maxDam = calculation.minDam + baseAction.APBonus * cp * AP
				end
			end
			if baseAction.DurationPerCombo then
				calculation.eDuration = calculation.eDuration + baseAction.DurationPerCombo * cp
			end
		else
			calculation.minDam = 0
			calculation.maxDam = 0
		end
	else
		if baseAction.APBonus then
			calculation.minDam = calculation.minDam + baseAction.APBonus * AP
			calculation.maxDam = calculation.maxDam + baseAction.APBonus * AP		
		end
	end
	
	if baseAction.Weapon and baseAction.Weapon ~= self:GetWeaponType() then
		calculation.minDam = 0
		calculation.maxDam = 0
	end
	
	if self.SetBonuses[playerClass] then
		self.SetBonuses[playerClass]( calculation, ActiveAuras )
	end		
	if self.Calculation[playerClass] then
		self.Calculation[playerClass]( calculation, ActiveAuras, BuffTalentRanks )
	end
	if self.SetBonuses[actionName] then
		self.SetBonuses[actionName]( calculation, ActiveAuras )
	end
	if self.Calculation[actionName] then
		self.Calculation[actionName]( calculation, ActiveAuras, BuffTalentRanks )
	end
	
	calculation.minDam = calculation.minDam * calculation.dmgM
	calculation.maxDam = calculation.maxDam * calculation.dmgM
	
	calculation.avg = ((calculation.minDam + calculation.maxDam) / 2)
	
	if not baseAction.NoCrits then
		if calculation.critPerc > 100 then
			calculation.critPerc = 100
		end

		calculation.avg = calculation.avg + (calculation.critPerc / 100) *  calculation.avg
	end
	
	calculation.avg = math.floor(calculation.avg + 0.5)
	calculation.minDam = math.floor(calculation.minDam + 0.5)
	calculation.maxDam = math.floor(calculation.maxDam + 0.5)
	calculation.AP = AP
	
	return calculation.avg, calculation.minDam, calculation.maxDam, calculation
end

DrD_TalentCalc = function( baseAction, talentName, talentRank, talent )
	local modType = talent.ModType
	local talentValue

	if type( talent.Effect ) == "table" then
		talentValue = talent.Effect[talentRank]
	else
		talentValue = talent.Effect * talentRank
	end		

	if not modType then
		if talent.Add then
			calculation.dmgM_AddTalent = calculation.dmgM_AddTalent + talentValue
		else
			calculation.dmgM = calculation.dmgM * ( 1 + talentValue )	
		end
	elseif modType == "Crit" then
		calculation.critPerc = calculation.critPerc + talentValue
	elseif modType == "Hit" then
		calculation.hitPerc = calculation.hitPerc + talentValue							
	elseif modType == "CritMultiplier" then
		calculation.critM = calculation.critM + talentValue
	elseif modType == "SpellDuration" then
		calculation.eDuration = calculation.eDuration + talentValue
	elseif modType == "Cooldown" then
		calculation.cooldown = calculation.cooldown - talentValue	
	elseif modType == "BuffTalentRanks" then
		BuffTalentRanks[talentName] = talentValue
	elseif modType == "FreeCrit" then
		calculation.freeCrit = talentValue
	end

	if modType then
		if DrDamage.Calculation[modType] then DrDamage.Calculation[modType]( calculation, talentValue ) end
	end				
end